==> rscheme_0.7.2.orig.tar.gz: rs-0.7.2/modules/mathlib/random.scm <==
#|------------------------------------------------------------*-Scheme-*--|
 | File:    modules/mathlib/random.scm
 |
 |          Copyright (C)1997 Donovan Kolbly <d.kolbly@rscheme.org>
 |          as part of the RScheme project, licensed for free use.
 |          See <http://www.rscheme.org/> for the latest information.
 |
 | File version:     1.5
 | File mod date:    1997.11.29 23:10:36
 | System build:     v0.7.2, 97.12.21
 | Owned by module:  mathlib
 |
 | Purpose:          (random) implementation
 `------------------------------------------------------------------------|#

(define-class <random-state> (<object>))

(define-glue (make-random-state seed)
 literals: ((& <random-state>))
{
  UINT_32 s;
  unsigned i;
  obj result;

  COUNT_ARGS(1);
  if (!OBJ_ISA_FIXNUM(seed))
    {
      scheme_error( "(make-random-state ~s): bad arg", 1, seed );
    }
  s = VAL(rehash_fixnum(seed));

  REG0 = result = alloc( SLOT(12), TLREF(0) );
  gvec_write_init_non_ptr( result, SLOT(0), int2fx(1) );
  for (i=0; i<11; i++)
    {
      gvec_write_init_non_ptr( result, 
			        SLOT(i+1), 
			        OBJ((s & ~PRIMARY_TAG_MASK) + FIXNUM_TAG) );
      s = ((s >> 1) & ~3) + (((s+4) & 8) ? 0x80000000 : 0);
      s = ((s >> 1) & ~3) + (((s+4) & 8) ? 0x80000000 : 0);
      s = ((s >> 1) & ~3) + (((s+4) & 8) ? 0x80000000 : 0);
    }
  RETURN1();
})
  
(define default-random-state
  (make-random-state 8901))

(define-glue (next-random state range)
 literals: ((& <random-state>))
{
  obj index, next_index, z;

  if (arg_count_reg == 2)
    {
      if (!OBJ_ISA_FIXNUM(range))
	scheme_error( "next-random: range arg ~s invalid", 1, range );
      else if (fx2int(range) < 1)
	scheme_error( "next-random: range arg ~d out of range", 1, range );
    }
  else if (arg_count_reg != 1)
    scheme_error( "next-random: ~d args, expected 1 or 2", 
		  1, int2fx(arg_count_reg) );

  if (!OBJ_ISA_PTR_OF_CLASS(state,TLREF(0)))
    {
      scheme_error( "next-random: state arg ~s invalid", 1, state );
    }

  index = gvec_read( state, SLOT(0) );
  assert( OBJ_ISA_FIXNUM(index) );

  next_index = ADD1(index);
  if (FXWORDS_TO_RIBYTES(next_index) >= SIZEOF_PTR(state))
    next_index = int2fx(1);
  z = FX_ADD( gvec_read( state, FXWORDS_TO_RIBYTES(index) ),
	      gvec_read( state, FXWORDS_TO_RIBYTES(next_index) ) );
  z = OBJ(VAL(z) & 0x7FFFFFFF);
  assert( OBJ_ISA_FIXNUM(z) );

  gvec_write_non_ptr( state, FXWORDS_TO_RIBYTES(index), z );
  gvec_write_non_ptr( state, SLOT(0), next_index );

  if (arg_count_reg == 2)
    z = OBJ(VAL(z) % VAL(range));
  REG0 = z;
  RETURN1();
})

#|
  (let* (((i1 <fixnum>) (gvec-ref state 0))
	 ((i2 <fixnum>) (add1 i1))
	 ((i2 <fixnum>) (if (eq? i2 (gvec-length state))
			    1
			    i2)))
    (let ((z (bitwise-and (fixnum+ (gvec-ref state i1)
				   (gvec-ref state i2))
			  #x1fffffff)))
      (gvec-set! state i1 z)
      (gvec-set! state 0 i2)
      z))
|#

(define (random . args)
  (apply* default-random-state args next-random))

==> rscheme_0.7.3pre.1.b39.orig.tar.gz: rs-0.7.3.1-b39/modules/mathlib/random.scm <==
#|------------------------------------------------------------*-Scheme-*--|
 | File:    modules/mathlib/random.scm
 |
 |          Copyright (C)1997 Donovan Kolbly <d.kolbly@rscheme.org>
 |          as part of the RScheme project, licensed for free use.
 |          See <http://www.rscheme.org/> for the latest information.
 |
 | File version:     1.6
 | File mod date:    1998.12.28 10:26:31
 | System build:     v0.7.3.1-b39, 1999-12-25
 | Owned by module:  mathlib
 |
 | Purpose:          (random) implementation
 `------------------------------------------------------------------------|#

(define-class <random-state> (<object>))

(define-glue (make-random-state seed)
 literals: ((& <random-state>))
{
  UINT_32 s;
  unsigned i;
  obj result;

  COUNT_ARGS(1);
  if (!OBJ_ISA_FIXNUM(seed))
    {
      scheme_error( "(make-random-state ~s): bad arg", 1, seed );
    }
  s = VAL(rehash_fixnum(seed));

  REG0 = result = alloc( SLOT(12), TLREF(0) );
  gvec_write_init_non_ptr( result, SLOT(0), int2fx(1) );
  for (i=0; i<11; i++)
    {
      gvec_write_init_non_ptr( result, 
			        SLOT(i+1), 
			        OBJ((s & ~PRIMARY_TAG_MASK) + FIXNUM_TAG) );
      s = ((s >> 1) & ~3) + (((s+4) & 8) ? 0x80000000 : 0);
      s = ((s >> 1) & ~3) + (((s+4) & 8) ? 0x80000000 : 0);
      s = ((s >> 1) & ~3) + (((s+4) & 8) ? 0x80000000 : 0);
    }
  RETURN1();
})
  
(define default-random-state
  (make-random-state 8901))

(define-glue (next-random state range)
 literals: ((& <random-state>))
{
  obj index, next_index, z;

  if (arg_count_reg == 2)
    {
      if (!OBJ_ISA_FIXNUM(range))
	scheme_error( "next-random: range arg ~s invalid", 1, range );
      else if (fx2int(range) < 1)
	scheme_error( "next-random: range arg ~d out of range", 1, range );
    }
  else if (arg_count_reg != 1)
    scheme_error( "next-random: ~d args, expected 1 or 2", 
		  1, int2fx(arg_count_reg) );

  if (!OBJ_ISA_PTR_OF_CLASS(state,TLREF(0)))
    {
      scheme_error( "next-random: state arg ~s invalid", 1, state );
    }

  index = gvec_read( state, SLOT(0) );
  assert( OBJ_ISA_FIXNUM(index) );

  next_index = ADD1(index);
  if (FXWORDS_TO_RIBYTES(next_index) >= SIZEOF_PTR(state))
    next_index = int2fx(1);
  z = FX_ADD( gvec_read( state, FXWORDS_TO_RIBYTES(index) ),
	      gvec_read( state, FXWORDS_TO_RIBYTES(next_index) ) );
  z = OBJ(VAL(z) & 0x7FFFFFFF);
  assert( OBJ_ISA_FIXNUM(z) );

  assert( OBJ_ISA_FIXNUM(next_index) );
  gvec_write_non_ptr( state, FXWORDS_TO_RIBYTES(index), z );
  gvec_write_non_ptr( state, SLOT(0), next_index );

  if (arg_count_reg == 2)
    z = OBJ(VAL(z) % VAL(range));
  REG0 = z;
  RETURN1();
})

#|
  (let* (((i1 <fixnum>) (gvec-ref state 0))
	 ((i2 <fixnum>) (add1 i1))
	 ((i2 <fixnum>) (if (eq? i2 (gvec-length state))
			    1
			    i2)))
    (let ((z (bitwise-and (fixnum+ (gvec-ref state i1)
				   (gvec-ref state i2))
			  #x1fffffff)))
      (gvec-set! state i1 z)
      (gvec-set! state 0 i2)
      z))
|#

(define (random . args)
  (apply* default-random-state args next-random))
